package de.privacynerd.chatbeautifier.displaynamecolor;

import de.privacynerd.chatbeautifier.ChatBeautifier;
import de.privacynerd.chatbeautifier.utils.ConfigFilenames;
import de.privacynerd.chatbeautifier.utils.DisplayNameSetter;
import de.privacynerd.chatbeautifier.utils.FileConfig;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;

public class ChatColorCommand implements CommandExecutor {
    @Override
    public boolean onCommand(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String alias, @NotNull String[] args) {
        ChatBeautifier.INSTANCE.log("Command /set-chat-name executed.");

        if(!(commandSender instanceof Player)) {
            ChatBeautifier.INSTANCE.log("This command was not executed by a player. Not doing anything.");
            return false;
        }
        Player player = (Player) commandSender;
        String validArgument = "";
        Component newName;
        int colorCodesListIndex;  // stores the index of the color code which was requested (0 for black color code, ...)

        // check for the right args
        if(args.length == 1) {
            switch (args[0]) {
                case "black":
                    validArgument = "black";
                    colorCodesListIndex = 0;
                    break;
                case "gray":
                    validArgument = "gray";
                    colorCodesListIndex = 1;
                    break;
                case "light-gray":
                    validArgument = "light-gray";
                    colorCodesListIndex = 2;
                    break;
                case "white":
                    validArgument = "white";
                    colorCodesListIndex = 3;
                    break;
                case "brown":
                    validArgument = "brown";
                    colorCodesListIndex = 4;
                    break;
                case "red":
                    validArgument = "red";
                    colorCodesListIndex = 5;
                    break;
                case "orange":
                    validArgument = "orange";
                    colorCodesListIndex = 6;
                    break;
                case "yellow":
                    validArgument = "yellow";
                    colorCodesListIndex = 7;
                    break;
                case "light-green":
                    validArgument = "light-green";
                    colorCodesListIndex = 8;
                    break;
                case "dark-green":
                    validArgument = "dark-green";
                    colorCodesListIndex = 9;
                    break;
                case "cyan":
                    validArgument = "cyan";
                    colorCodesListIndex = 10;
                    break;
                case "blue":
                    validArgument = "blue";
                    colorCodesListIndex = 11;
                    break;
                case "purple":
                    validArgument = "purple";
                    colorCodesListIndex = 12;
                    break;
                case "magenta":
                    validArgument = "magenta";
                    colorCodesListIndex = 13;
                    break;
                case "pink":
                    validArgument = "pink";
                    colorCodesListIndex = 14;
                    break;
                case "list":
                    ChatBeautifier.INSTANCE.log("Got valid argument 'list', printing list of valid colors...");
                    ArrayList<String> colorNames = ChatColorUtils.getChooseColorStrings();

                    // now compose the message containing all possible color args
                    Component message = Component.text("Mögliche Farben: ")
                            .decoration(TextDecoration.BOLD, true);

                    String newMessageItem = colorNames.get(0).toLowerCase().replace(" ", "-");
                    message = message.append(Component.text(newMessageItem, NamedTextColor.GRAY)
                            .decoration(TextDecoration.BOLD, false)
                            .decoration(TextDecoration.ITALIC, true));

                    for (int i=1; i<colorNames.toArray().length; i++) {
                        newMessageItem = ", " + colorNames.get(i).toLowerCase().replace(" ", "-");
                        message = message.append(Component.text(newMessageItem, NamedTextColor.GRAY)
                                .decoration(TextDecoration.BOLD, false)
                                .decoration(TextDecoration.ITALIC, true));
                    }

                    // and send it
                    ChatBeautifier.INSTANCE.tellPlayer(message, player);
                    return true;
                case "gui":
                    ChatBeautifier.INSTANCE.log("Got valid argument 'gui', opening gui...");
                    player.openInventory(new ChooseChatColorGUI().getGUI(player));
                    return true;
                default:
                    // print a help string if no valid argument is given
                    ChatBeautifier.INSTANCE.tellPlayer("Verwendung: /" + alias + " [<color>, list, gui]", player);
                    return true;
            }

            // save the selected color to the file
            FileConfig chatColorConfig = new FileConfig(ConfigFilenames.chatNameColors);
            chatColorConfig.set(String.valueOf(player.getUniqueId().toString()), colorCodesListIndex);
            chatColorConfig.saveConfig();

            // update name (this method respects color values in the file
            DisplayNameSetter.setName(player);

            ChatBeautifier.INSTANCE.log("Got valid color " + validArgument);
            ChatBeautifier.INSTANCE.tellPlayer(Component.text("Dein neuer Name: ")
                    .append(player.displayName()
                    ), player);
            return true;
        } else if (args.length > 1) {  // print a help string if more than one argument is given
            ChatBeautifier.INSTANCE.tellPlayer("Verwendung: /" + alias + " [<color>, list, gui]", player);
            return true;
        } else {
            player.openInventory(new ChooseChatColorGUI().getGUI(player));
            return true;
        }
    }
}
